# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl test.pl'

#########################

# change 'tests => 1' to 'tests => last_test_to_print';

use Test;
use Cwd;

$testid = 0;

BEGIN { plan tests => 18 };

use lib '../Base/blib/lib';	# snag Datastore::Base if not installed...
use Datastore::MD5;

ok(++$testid); # If we made it this far, we're ok.

#########################

sub Not {
  print "not ";
}

###########################################################################
# create a datastore object.
print "#################\n# Test basic functionality.\n#\n";
$ds = new Datastore::MD5("root" => "/tmp/foo", "depth" => 2);
Not() if (!defined($ds)); ok(++$testid);

Not() if ($ds->version() ne "0.11"); ok(++$testid);

# make sure that it has the characteristics we expect.
Not() if ($ds->depth() != 2); ok(++$testid);
Not() if ($ds->root() ne "/tmp/foo/"); ok(++$testid); # watch out for trailing "/"

# try to set some characteristics after creation.
Not() if ($ds->depth(5) != 5); ok(++$testid);
Not() if ($ds->depth() != 5); ok(++$testid);
Not() if ($ds->root("/tmp") ne "/tmp/"); ok(++$testid); # watch out for trailing "/"
Not() if ($ds->root() ne "/tmp/"); ok(++$testid); # watch out for trailing "/"

###########################################################################
# start over.
print "#################\n# Test mkdir success and failure.\n#\n";
$ds = new Datastore::MD5("root" => "/tmp/datastore-test-$$", "depth" => 2);

# create a datastore dir, and check that it exists.
Not() if (!$ds->mkdir("CG1234")); ok(++$testid);
Not() if (! -x "/tmp/datastore-test-$$/EF/34/CG1234/"); ok(++$testid);

# check a failure mode.
# this shouldn't work since the file gets in the way of the mkdir....
system("touch /tmp/datastore-broken-$$");
$ds->root("/tmp/datastore-broken-$$");
Not() if ($ds->mkdir("CG1234")); ok(++$testid); # expect failure.

###########################################################################
# start over.
print "#################\n# Test pushd and popd.\n#\n";
$ds = new Datastore::MD5("root" => "/tmp/datastore-test-$$", "depth" => 2);
$cwd = getcwd();
$ds->pushd("CG1040");
# watch trailing "/", getcwd doesn't include it!
Not() if (getcwd() !~ m|.*datastore-test-$$/0A/70/CG1040|); ok(++$testid);
$ds->popd();
Not() if (getcwd() ne $cwd); ok(++$testid);

print "#################\n# Test removing directories.\n#\n";
$ds->rmdir("CG1040");
Not() if (-e $ds->id_to_dir("CG1040")); ok(++$testid);

###########################################################################
# start over.
print "#################\n# Test readonly access.\n#\n";
$ds = new Datastore::MD5("root" => "/tmp/datastore-test-$$", "depth" => 2);

# this should work.
Not() if (!$ds->mkdir("CG9876")); ok(++$testid);

# make it readonly
$ds->readonly(1);                  

# and we expect this to fail.
Not() if ($ds->mkdir("CG9875")); ok(++$testid);

# make it read-write
$ds->readonly(0);                  

# and now it should work.
Not() if (!$ds->mkdir("CG9875")); ok(++$testid);

system("rm -rf /tmp/foo /tmp/datastore-broken-$$ /tmp/datastore-test-$$");
